from bs4 import BeautifulSoup
from PIL import Image, ExifTags
from datetime import datetime
import os
import math
import zipfile
import random
from dateutil.relativedelta import relativedelta


def dither_image(image_path):
    image = Image.open(image_path)
    try:
        for orientation in ExifTags.TAGS.keys():
            if ExifTags.TAGS[orientation] == 'Orientation':
                break
        exif = dict(image._getexif().items())
        if exif[orientation] == 3:
            image = image.rotate(180, expand=True)
        elif exif[orientation] == 6:
            image = image.rotate(270, expand=True)
        elif exif[orientation] == 8:
            image = image.rotate(90, expand=True)
    except (AttributeError, KeyError, IndexError):
        pass
    
    # image.thumbnail(max_size, Image.ANTIALIAS)  # This line is commented out.
    
    if image.mode == 'RGBA':
        r, g, b, a = image.split()
        rgb_image = Image.merge('RGB', (r, g, b))
        dithered_image = rgb_image.convert('1')
        final_image = Image.merge('LA', (dithered_image, a))
    else:
        final_image = image.convert('1')
    
    return final_image

def convert_size(size_bytes):
    if size_bytes == 0:
        return "0B"
    size_name = ("B", "KB", "MB", "GB")
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, 2)
    return f"{s} {size_name[i]}"

if not os.path.exists('dithered_images'):
    os.makedirs('dithered_images')

with open('hires.html', 'r', encoding='utf-8') as file:
    soup = BeautifulSoup(file, 'html.parser')

img_tags = soup.find_all('img')

for img in img_tags:
    src = img['src']
    original_extension = os.path.splitext(src)[1].lower()
    
    if original_extension == '.png':
        new_image_path = os.path.join('dithered_images', os.path.splitext(os.path.basename(src))[0] + '.png')
        save_format = 'PNG'
        save_params = {'compress_level': 9}
    else:
        new_image_path = os.path.join('dithered_images', os.path.splitext(os.path.basename(src))[0] + '.jpg')
        save_format = 'JPEG'
        save_params = {'quality': 10}
    
    if os.path.exists(src):
        if not os.path.exists(new_image_path):
            dithered_image = dither_image(src)
            dithered_image.save(new_image_path, save_format, **save_params)  
        
        img['src'] = new_image_path
        img['data-hires'] = src  # Store the original file path
        file_size = os.path.getsize(new_image_path)
        file_size_human = convert_size(file_size)
        img['data-size'] = file_size_human

with open('index.html', 'w', encoding='utf-8') as file:
    file.write(str(soup))

print('HTML updated successfully with dithered images.')




# ////// GENERATE README FILE //////


# Get the current date and time
now = datetime.now()

# Format the date and time
formatted_date = now.strftime("%B %d, %Y")
formatted_time = now.strftime("%H:%M")
day_of_week = now.strftime("%A")

# Calculate the date and time 10 years from now
date_time_10_years_later = now + relativedelta(years=10)
formatted_date_time_10_years_later = date_time_10_years_later.strftime("%B %d, %Y at %H:%M")

# Determine the season (simplistic approach)
month = now.month
if 3 <= month < 6:
    season = "Spring"
elif 6 <= month < 9:
    season = "Summer"
elif 9 <= month < 12:
    season = "Autumn"
else:
    season = "Winter"

# Generate a random color in hex format
random_color = f'#{random.randint(0, 0xFFFFFF):06x}'

# Specify the content to be written to the file, replacing placeholders with current info
content = f"""This version of www.filelife.tours is from {formatted_date} at {formatted_time} CEST ({season}). The File Life project was created by Elliott Cost (email@elliott.computer) and made possible through the generous support of USB Club.

Second: {now.second}
Day of week: {day_of_week}
Day: {now.day}
Month: {now.strftime('%B')}
Year: {now.year}
Season: {season}
Color: {random_color}

Place this archived version of the website on a USB and store in a cool dry place. Contact Elliott in 10 years ({formatted_date_time_10_years_later}) to say that you have this version of File Life website. Thank you!
"""

# Open a file named 'readme.txt' in the same directory for writing
# The 'with' statement ensures that the file is automatically closed after writing
with open('readme.txt', 'w') as file:
    file.write(content)




# ////////// ZIP THE SITE ///////////

import zipfile
import os

def zip_directory(directory_path, output_zip, exclude_files=None, exclude_dirs=None):
    """
    Zips the contents of directory_path into a zip file output_zip, excluding specific files and directories.

    :param directory_path: Path of the directory where the files to be zipped are located
    :param output_zip: Path of the output zip file
    :param exclude_files: List of file names to exclude
    :param exclude_dirs: List of directory names to exclude
    """
    if exclude_files is None:
        exclude_files = []
    if exclude_dirs is None:
        exclude_dirs = []

    with zipfile.ZipFile(output_zip, 'w') as zipf:
        for root, dirs, files in os.walk(directory_path):
            # Exclude directories
            dirs[:] = [d for d in dirs if d not in exclude_dirs]

            for file in files:
                # Exclude files
                if file not in exclude_files:
                    file_path = os.path.join(root, file)
                    zipf.write(file_path, os.path.relpath(file_path, os.path.join(directory_path, '..')))

# Example usage
directory_to_zip = os.getcwd()  # Gets the current working directory
output_zip_file = os.path.join(directory_to_zip, "file-life-dot-tours-website.zip")  # Creates the output path in the current directory

files_to_exclude = [".DS_Store", "notes.txt", "file-life-dot-tours-website.zip", "build.txt"]  # Replace with your file names to exclude
dirs_to_exclude = ["_archived", ".vscode", "posts"]  # Replace with your directory names to exclude

zip_directory(directory_to_zip, output_zip_file, exclude_files=files_to_exclude, exclude_dirs=dirs_to_exclude)

zip_file_size = convert_size(os.path.getsize(output_zip_file))

# Re-open the newly created index.html to add the zipped file size
with open('index.html', 'r', encoding='utf-8') as file:
    soup = BeautifulSoup(file, 'html.parser')

# Find the span with id 'zip-size' and update its contents
zip_size_span = soup.find('span', {'id': 'zip-size'})
now = datetime.now()
formatted_now = now.strftime("%b %d, %H:%M")
if zip_size_span:
    zip_size_span.string = f"({zip_file_size})"
else:
    print("Warning: No <span> with id 'zip-size' found in index.html")
    
# Find the span with id 'time-generated' and update its contents
time_generated_span = soup.find('span', {'id': 'time-generated'})
now = datetime.now()
formatted_now = now.strftime("%b %d, %H:%M")
if time_generated_span:
    time_generated_span.string = f"This download was created on {formatted_now} CEST."
else:
    print("Warning: No <span> with id 'time-generated' found in index.html")

# Save the modified HTML back to index.html
with open('index.html', 'w', encoding='utf-8') as file:
    file.write(str(soup))
